/*
 * Created by: 
 * Created: Saturday, March 24, 2007
 */

using System;
using System.Data;
using System.Data.SqlClient;
using CogArtTech.Music.Library.Lists;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library
{
    [Serializable]
    public class Band : BusinessBase<Band>
    {
        private string _Name = string.Empty;
        private SmartDate _StartDate = new SmartDate();
        private SmartDate _EndDate = new SmartDate();
        private int _Id = 0;
        private BandSongsList _Songs = null;
        private string _CountryCode = string.Empty;
        private BandMembersList _Members = null;

        private Band()
        {
            MarkAsChild();
        }

        private Band(SafeDataReader dr)
        {
            MarkAsChild();
            Fetch(dr);
        }

        internal static Band NewBand()
        {
            Band band = new Band();
            band.MarkNew();
            return band;
        }

        public static Band GetBand()
        {
            return new Band();
        }

        public static Band GetBand(SafeDataReader dr)
        {
            return new Band(dr);
        }

        protected override object GetIdValue()
        {
            return _Id;
        }

        public string Name
        {
            get { return _Name; }
            set
            {
                if (_Name != value)
                {
                    _Name = value;
                    PropertyHasChanged();
                }
            }
        }

        public SmartDate StartDate
        {
            get { return _StartDate; }
            set
            {
                if (_StartDate != value)
                {
                    _StartDate = value;
                    PropertyHasChanged();
                }
            }
        }

        public SmartDate EndDate
        {
            get { return _EndDate; }
            set
            {
                if (_EndDate != value)
                {
                    _EndDate = value;
                    PropertyHasChanged();
                }
            }
        }

        public int Id
        {
            get { return _Id; }
            set { _Id = value; }
        }

        public string CountryCode
        {
            get { return _CountryCode; }
            set
            {
                if (_CountryCode != value)
                {
                    _CountryCode = value;
                    PropertyHasChanged();
                }
            }
        }

        public BandSongsList Songs
        {
            get { return _Songs; }
        }

        public BandMembersList Members
        {
            get { return _Members; }
        }

        internal void DeleteSelf()
        {
            // if we're new then don't update the database
            if (this.IsNew) return;

            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cmd = new SqlCommand("usp_DeleteBand", cn))
                {
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("band_id", Id);

                    cmd.ExecuteNonQuery();
                }

                MarkNew();
            }
        }

        internal void Update()
        {
            if (base.IsDirty)
            {
                using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
                {
                    cn.Open();
                    using (SqlCommand cmd = new SqlCommand("usp_UpdateBand", cn))
                    {
                        cmd.Connection = cn;
                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.Parameters.AddWithValue("band_id", _Id);
                        cmd.Parameters.AddWithValue("name", _Name);
                        cmd.Parameters.AddWithValue("start_date", _StartDate.Date);
                        cmd.Parameters.AddWithValue("end_date", _EndDate.Date);
                        cmd.ExecuteNonQuery();
                    }
                }
            }


            if (this.Members.IsDirty)
            {
                this.Members.Update(this);
            }
            if (this.Songs.IsDirty)
            {
                this.Songs.Update(this);
            }


            MarkClean();
        }

        internal void Insert()
        {
            // if we're not dirty then don't update the database
            if (!this.IsDirty) return;

            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cmd = new SqlCommand("usp_InsertBand", cn))
                {
                    cmd.Connection = cn;
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("band_id", _Id);
                    cmd.Parameters.AddWithValue("name", _Name);
                    cmd.Parameters.AddWithValue("start_date", _StartDate);
                    cmd.Parameters.AddWithValue("end_date", _EndDate);
                    cmd.ExecuteNonQuery();

                    SqlParameter param =
                        new SqlParameter("@band_id", SqlDbType.Int);
                    param.Direction = ParameterDirection.Output;
                    cmd.Parameters.Add(param);
                    cmd.ExecuteNonQuery();
                    this.Id = Convert.ToInt32(cmd.Parameters["@band_id"].Value);
                    MarkOld();
                    MarkClean();
                }
                MarkOld();
            }

            this.Members.Update(this);
            this.Songs.Update(this);
        }

        private void Fetch(SafeDataReader dr)
        {
            _Id = dr.GetInt32("band_id");
            _Name = dr.GetString("name");
            _CountryCode = dr.GetString("country");
            if (dr["start_date"] != DBNull.Value)
            {
                _StartDate = new SmartDate(Convert.ToDateTime(dr["start_date"]));
            }
            else
            {
                _StartDate = new SmartDate();
            }

            if (dr["end_date"] != DBNull.Value)
            {
                _EndDate = new SmartDate(Convert.ToDateTime(dr["end_date"]));
            }
            else
            {
                _EndDate = new SmartDate();
            }
            MarkOld();
            //get members

            _Members = BandMembersList.GetList(this.Id);

            //get songs
            _Songs = BandSongsList.GetList(this);
        }
    }
}