/*
 * Created by: 
 * Created: Saturday, March 24, 2007
 */

using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library.Lists
{
    [Serializable]
    public class BandMembersList : BusinessListBase<BandMembersList, Musician>
    {
        #region Business Methods

        public static BandMembersList GetList(int bandId)
        {
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cm = cn.CreateCommand())
                {
                    cm.CommandType = CommandType.StoredProcedure;
                    cm.CommandText = "usp_SelectMusicianByBand";
                    cm.Parameters.AddWithValue("@band_id", bandId);

                    using (SafeDataReader dr = new SafeDataReader(cm.ExecuteReader()))
                    {
                        return GetList(dr);
                    }
                }
            }
        }

        public Musician GetItem(int musicianId)
        {
            foreach (Musician musician in this)
                if (musician.Id == musicianId)
                    return musician;
            return null;
        }

        public void Assign(Musician musician)
        {
            Assign(musician.Id);
        }

        public void Assign(int musicianId)
        {
            if (!Contains(musicianId))
            {
                foreach (Musician musician in MusiciansList.GetList())
                {
                    if (musician.Id == musicianId)
                    {
                        Musician newMusician = Musician.NewMusician();
                        newMusician.Id = musicianId;
                        newMusician.FirstName = musician.FirstName;
                        newMusician.LastName = musician.LastName;
                        newMusician.DateOfBirth = musician.DateOfBirth;
                        this.Add(newMusician);
                    }
                }
            }
            else
                throw new InvalidOperationException(
                    "Musician already assigned to band");
        }

        public void Unassign(Musician musician)
        {
            Unassign(musician.Id);
        }

        public void Unassign(int musicianId)
        {
            Remove(musicianId);
        }

        public void Remove(int musicianId)
        {
            foreach (Musician musician in this)
            {
                if (musician.Id == musicianId)
                {
                    Remove(musician);
                    break;
                }
            }
        }

        public bool Contains(int musicianId)
        {
            foreach (Musician musician in this)
                if (musician.Id == musicianId)
                    return true;
            return false;
        }

        public bool ContainsDeleted(int musicianId)
        {
            foreach (Musician musician in DeletedList)
                if (musician.Id == musicianId)
                    return true;
            return false;
        }

        internal void Update(Band band)
        {
            this.RaiseListChangedEvents = false;
            // update (thus deleting) any deleted child objects
            foreach (Musician obj in DeletedList)
            {
                RemoveMemberFromBand(band, obj.Id);
            }
            DeletedList.Clear();

            // Assign 
            foreach (Musician obj in this)
            {
                if (obj.IsNew)
                {
                    obj.Assign(band);
                }
  
            }
            this.RaiseListChangedEvents = true;
        }

        private new void Add(Musician musician)
        {
            base.Add(musician);
        }

        #endregion

        #region Factory Methods

        internal static BandMembersList NewMusiciansList()
        {
            return new BandMembersList();
        }

        internal static BandMembersList GetList(SafeDataReader dr)
        {
            return new BandMembersList(dr);
        }

        private BandMembersList()
        {
            MarkAsChild();
        }

        private BandMembersList(SafeDataReader dr)
        {
            MarkAsChild();
            Fetch(dr);
        }

        #endregion

        #region Data Access

        // called to load data from the database
        private void Fetch(SafeDataReader dr)
        {
            this.RaiseListChangedEvents = false;
            this.Clear();
            while (dr.Read())
                this.Add(Musician.GetMusician(dr));
            this.RaiseListChangedEvents = true;
        }

        private void RemoveMemberFromBand(Band band, int musicianId)
        {
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                SqlCommand cmd = new SqlCommand();
                cmd.Connection = cn;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.CommandText = "usp_DeleteMusicianByBand";
                cmd.Parameters.AddWithValue("@musician_id", musicianId);
                cmd.Parameters.AddWithValue("@band_id", band.Id);

                cmd.ExecuteNonQuery();
            }
        }

        #endregion
    }
}