/*
 * Created by: 
 * Created: Sunday, March 25, 2007
 */

using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library.Lists
{
    [Serializable]
    public class MusiciansList : BusinessListBase<MusiciansList, Musician>
   {
      private static MusiciansList _List;

      #region Business Methods

      public Musician GetItem(int musicianId)
      {
         foreach (Musician musician in this)
            if (musician.Id == musicianId)
               return musician;
         return null;
      }

      private void Unassign(Musician musician)
      {
          Unassign(musician.Id);
      }

      public void Unassign(int musicianId)
      {

      }

      public void Assign(int musicianId)
      {
         if (!Contains(musicianId))
         {
            Musician musician =
              Musician.NewMusician();
            this.Add(musician);
         } else
            throw new InvalidOperationException(
              "Musician already assigned to List");
      }

      public void Remove(int musicianId)
      {
         foreach (Musician musician in this)
         {
            if (musician.Id == musicianId)
            {
               Remove(musician);
               break;
            }
         }
      }

      public bool Contains(int musicianId)
      {
         foreach (Musician musician in this)
            if (musician.Id == musicianId)
               return true;
         return false;
      }

      public bool ContainsDeleted(int musicianId)
      {
         foreach (Musician musician in DeletedList)
            if (musician.Id == musicianId)
               return true;
         return false;
      }

      internal static void InvalidateCache()
      {
         _List = null;
      }

      #endregion

      #region Factory Methods

      public static MusiciansList NewMusiciansList()
      {

         return new MusiciansList();
      }

      public static MusiciansList GetList()
      {
         if (_List == null)
         {
            _List = DataPortal.Fetch<MusiciansList>(new Criteria());
         }
         return _List;
      }

      private MusiciansList()
      {
         
      }

      private MusiciansList(SafeDataReader dr)
      {
         
         Fetch(dr);
      }

      #endregion

      #region Data Access

      // called to load data from the database
      private void Fetch(SafeDataReader dr)
      {
         this.RaiseListChangedEvents = false;
         while (dr.Read())
            this.Add(Musician.GetMusician(dr));
         this.RaiseListChangedEvents = true;
      }

        protected override void DataPortal_Update()
        {
            Update();
        }
        
        internal void Update()
        {
            this.RaiseListChangedEvents = false;
            // update (thus deleting) any deleted child objects
            foreach (Musician obj in DeletedList)
                obj.DeleteSelf();  

            // now that they are deleted, remove them from memory too
            DeletedList.Clear();

            // add/update any current child objects
            foreach (Musician obj in this)
            {
                if (obj.IsNew)
                {
                    obj.Insert();
                }

                if (obj.IsDirty && !obj.IsNew)
                {
                    obj.Update();
                }

            }
            this.RaiseListChangedEvents = true;
        }

      private void DataPortal_Fetch(Criteria criteria)
      {
         using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
         {
            cn.Open();
            using (SqlCommand cm = cn.CreateCommand())
            {
               cm.CommandType = CommandType.StoredProcedure;
               cm.CommandText = "GetMusicians";

               using (SafeDataReader dr = new SafeDataReader(cm.ExecuteReader()))
               {
                  while(dr.Read())
                  {
                      Musician musician = Musician.GetMusician(dr);
                      Add(musician);
                  }
               }
            }
         }
      }
        
      public static Musician NewMusician()
      {
          return Musician.NewMusician();
      }

      #endregion

      private class Criteria
      {
      }

   }
}