using System;
using System.Collections.Generic;
using System.Text;
using CogArtTech.Music.Library;
using CogArtTech.Music.Library.Lists;
using NUnit.Framework;
using Csla;

namespace CogArtTech.Music.Tests
{
    [TestFixture]
    public class LibraryTest
    {
        [Test]
        public void BandInitTest()
        {
            Band band = Band.GetBand();
        }

        [Test]
        public void BandsListTest()
        {
            BandsList bands = BandsList.GeList();
            Assert.Greater(bands.Count,0);
        }

        [Test]
        public void SongsListTest()
        {
            SongsList list = SongsList.GetList();
            Assert.Greater(list.Count, 0);
        }
        [Test]
        public void MusiciansListTest()
        {
            MusiciansList list = MusiciansList.GetList();
            Assert.Greater(list.Count, 0);
        }
        
        [Test]
        public void InstrumentListTest()
        {
            InstrumentsList list = InstrumentsList.GetList();
            Assert.Greater(list.Count,0);
        }

        [Test]
        public void CatalogTest()
        {
            //Get an instance of a Catalog object by
            //calling the static GetCatalog(string countryOfBand) 
            //creation method. NOTE: The Catalog object contains a
            //Bands property, which is a collection of Band objects.
            Catalog catalog = Catalog.NewCatalog("UK");
            BandsList list = catalog.Bands;
            Assert.Greater(list.Count, 0);
        }
        
        [Test]
        public void SongSaveTest()
    {
        Song song = SongsList.NewSong("Paint it, Black", 135);
        SongsList list = SongsList.GetList();
        list.Add(song);
        list.Save();

        SongsList buffer = SongsList.GetList();
        bool hasSong = false;
        foreach(Song s in buffer)
        {
            if(s.Name=="Paint it, Black")
            {
                hasSong = true;
                break;
            }
        }
        Assert.IsTrue(hasSong);
    }

        [Test]
        public void SongUpdateTest()
    {
        string oldSongName = "Paint it, Black";
        string newSongName = "Paint It, Black";

        SongsList list = SongsList.GetList();
        int updatedSongId = 0;
        foreach (Song song in list)
        {
            if (song.Name == oldSongName)
            {
                song.Name = newSongName;
                updatedSongId = song.Id;
                break;
            }
        }
        bool b = false;
        if(updatedSongId>0)
        {
            list.Save();
            list = SongsList.GetList();
            foreach (Song song in list)
            {
                if (song.Id == updatedSongId)
                {
                    b = (song.Name == newSongName);
                    break;
                }
            } 
        }
        Assert.IsTrue(b);
    }

        [Test]
        public void InstrumentSaveTest()
        {
            string newDescription = "piccolo";
            Instrument instrument = InstrumentsList.NewInstrument();
            instrument.Description = newDescription;
            InstrumentsList instruments = InstrumentsList.GetList();
            instruments.Add(instrument);
            instruments.Save();
            InstrumentsList buffer = InstrumentsList.GetList();
            bool hasInstrument = false;
            foreach (Instrument inst in buffer)
            {
                if (inst.Description == newDescription)
                {
                    hasInstrument = true;
                    break;
                }
            }
            Assert.IsTrue(hasInstrument);
        }

        [Test]
        public void InstrumentUpdateTest()
        {
            string oldInstrumentName = "piccolo";
            string newInstrumentName = "Piccollo";

            InstrumentsList list = InstrumentsList.GetList();
            int updateInstrumentId = 0;
            foreach (Instrument instrument in list)
            {
                if (instrument.Description == oldInstrumentName)
                {
                    instrument.Description = newInstrumentName;
                    updateInstrumentId = instrument.Id;
                    break;
                }
            }
            bool b = false;
            if (updateInstrumentId > 0)
            {
                list.Save();
                InstrumentsList bufffer = InstrumentsList.GetList();
                foreach (Instrument instrument in bufffer)
                {
                    if (instrument.Id == updateInstrumentId)
                    {
                        b = (instrument.Description == newInstrumentName);
                        break;
                    }
                }
            }
            Assert.IsTrue(b);
        }
        
        [Test]
        public void AddMemberToBandTest()
        {

            Catalog catalog = Catalog.GetCatalog("UK");

            int musicianIdAdded = 0;
            foreach (Band band in catalog.Bands)
            {
                if (band.Name == "The Beatles")
                {
                    foreach (Musician musician in MusiciansList.GetList())
                    {
                        if (musician.LastName == "Harrison")
                        {
                            musicianIdAdded = musician.Id;
                            
                            band.Members.Assign(musician);
                            catalog.Save();
                            break;
                        }
                    }
                    break;
                }
            }

            catalog = Catalog.GetCatalog("USA");
            catalog = Catalog.GetCatalog("UK");

            bool isAdded = false;
            foreach (Band band in catalog.Bands)
            {
                if (band.Name == "The Beatles")
                {
                    foreach (Musician musician in MusiciansList.GetList())
                    {
                        if (musician.Id == musicianIdAdded)
                        {
                            isAdded = true;
                            break;
                        }
                    }
                    break;
                }
            }

            Assert.IsTrue(isAdded);
            
        }

        [Test]
        public void AddMemberToBandWithInstrumentTest()
        {
            Catalog catalog = Catalog.GetCatalog("UK");

            int musicianIdAdded = 0;
            foreach (Band band in catalog.Bands)
            {
                if (band.Name == "The Beatles")
                {
                    MusiciansList musicians = MusiciansList.GetList();
                    foreach (Musician musician in musicians)
                    {
                        if (musician.LastName == "Starr")
                        {
                            musicianIdAdded = musician.Id;

                            //add the drums
                            foreach (Instrument instrument in InstrumentsList.GetList())
                            {
                                if (instrument.Description.ToUpper() == "DRUMS")
                                {
                                    musician.Instruments.Assign(instrument);
                                    musicians.Save();
                                    break;
                                }

                            }

                            band.Members.Assign(musician);
                            
                           
                            catalog.Save();
                            break;
                        }
                    }
                    break;
                }
            }

            catalog = Catalog.GetCatalog("USA");
            catalog = Catalog.GetCatalog("UK");

            bool isAdded = false;
            foreach (Band band in catalog.Bands)
            {
                if (band.Name == "The Beatles")
                {
                    foreach (Musician musician in MusiciansList.GetList())
                    {
                        if (musician.Id == musicianIdAdded)
                        {
                            isAdded = true;
                            break;
                        }
                    }
                    break;
                }
            }

            Assert.IsTrue(isAdded);

        }
        
        [Test]
        public void RemoveMemberFromBand()
        {
            Catalog catalog = Catalog.GetCatalog("UK");
            Musician musicianToRemove = null;
            foreach(Band band in catalog.Bands)
            {
                foreach(Musician musician in band.Members)
                {
                    if(musician.LastName.ToLower()=="richards")
                    {
                        musicianToRemove = musician;
                        break;
                    }
                }

                if (musicianToRemove != null)
                {
                    band.Members.Unassign(musicianToRemove);
                }
            }

            catalog.Save();
            

        }
    
     
    }
}
